@@ -18,7 +18,7 @@
 extract_sparse_parts <- function(A) {
   if (!requireNamespace("Matrix")) 
     stop("You have to install the Matrix package to call 'extract_sparse_parts'")
-  if (class(A) == 'matrix') 
+  if (!is(A, 'Matrix')) 
     A <- Matrix::Matrix(A, sparse=TRUE)
   A <- Matrix::t(A)
   A <- as(A, "dgCMatrix")
