@@ -971,7 +971,7 @@ get_par_summary_msd <- function(sim, n) {
                  if (sim$warmup2[i] == 0) sim$samples[[i]][[n]]
                  else sim$samples[[i]][[n]][-(1:sim$warmup2[i])]
                })
-  sumfun <- function(chain) c(mean(chain, na.rm = TRUE), sd(chain, na.rm = TRUE))
+  sumfun <- function(chain) c(mean(chain), sd(chain))
   cs <- lapply(ss, sumfun)
   as <- sumfun(do.call(c, ss))
   list(msd = as, c_msd = unlist(cs, use.names = FALSE))
@@ -1025,6 +1025,7 @@ combine_msd_quan <- function(msd, quan) {
 
 
 summary_sim <- function(sim, pars, probs = default_summary_probs()) {
+  # cat("summary_sim is called.\n")
   probs_len <- length(probs)
   pars <- if (missing(pars)) sim$pars_oi else check_pars_second(sim, pars)
   tidx <- pars_total_indexes(sim$pars_oi, sim$dims_oi, sim$fnames_oi, pars)
@@ -1083,7 +1084,6 @@ summary_sim <- function(sim, pars, probs = default_summary_probs()) {
 }
 
 summary_sim_quan <- function(sim, pars, probs = default_summary_probs()) {
-  # cat("summary_sim is called.\n")
   probs_len <- length(probs)
   pars <- if (missing(pars)) sim$pars_oi else check_pars_second(sim, pars)
   tidx <- pars_total_indexes(sim$pars_oi, sim$dims_oi, sim$fnames_oi, pars)
@@ -1525,7 +1525,7 @@ read_csv_header <- function(f, comment.char = '#') {
       warmup.count <- 0L
     }
     if (grepl("#.*thin", input)){
-      thin <- as.integer(gsub("[^0-9]*([0-9]*).*", "\\1", input))
+      thin <- as.integer(gsub("[^0-9]*([0-9]*).*","\\1",input))
     }
     if (grepl("#.*save_warmup",input)){
       save.warmup <- !grepl("0",input)
@@ -1533,9 +1533,6 @@ read_csv_header <- function(f, comment.char = '#') {
     if (grepl("#.*output_sample",input)){
       iter.count <- as.numeric(gsub("[^0-9]*([0-9]*).*","\\1",input))
     }
-    if (!is.null(thin)) {
-      iter.count <- iter.count %/% thin
-    }
 
   }
   header <- input
@@ -1544,7 +1541,10 @@ read_csv_header <- function(f, comment.char = '#') {
       iter.count <- warmup.count + sample.count
     else
       iter.count <- sample.count
-  } 
+  }
+  if(!is.null(thin)){
+    iter.count <- iter.count %/% thin
+  }
   attr(header, "iter.count") <- iter.count
   attr(header, "lineno") <- niter
   close(con)
@@ -1708,3 +1708,8 @@ sample_indices <- function(wts, n_draws) {
   }
   return(idx)
 }
+
+test_221 <- function(cppcode) {
+  grepl("Code generated by Stan version 2.2", cppcode, fixed = TRUE) ||
+  grepl("Code generated by Stan version 3", cppcode, fixed = TRUE)  
+}
