@@ -83,6 +83,7 @@ z_scale <- function(x) {
   S <- length(x)
   r <- rank(x, ties.method = 'average')
   z <- qnorm((r - 1 / 2) / S)
+  z[is.na(x)] <- NA
   if (!is.null(dim(x))) {
     # output should have the input dimension
     z <- array(z, dim = dim(x), dimnames = dimnames(x))
@@ -107,6 +108,7 @@ u_scale <- function(x) {
   S <- length(x)
   r <- rank(x, ties.method = 'average')
   u <- (r - 1 / 2) / S
+  u[is.na(x)] <- NA
   if (!is.null(dim(x))) {
     # output should have the input dimension
     u <- array(u, dim = dim(x), dimnames = dimnames(x))
@@ -129,6 +131,7 @@ u_scale <- function(x) {
 r_scale <- function(x) {
   S <- length(x)
   r <- rank(x, ties.method = 'average')
+  r[is.na(x)] <- NA
   if (!is.null(dim(x))) {
     # output should have the input dimension
     r <- array(r, dim = dim(x), dimnames = dimnames(x))
@@ -168,10 +171,15 @@ is_constant <- function(x, tol = .Machine$double.eps) {
 #' localization: An improved R-hat for assessing convergence of
 #' MCMC. \emph{arXiv preprint} \code{arXiv:1903.08008}.
 rhat_rfun <- function(sims) {
-  if (any(!is.finite(sims)))
+  if (anyNA(sims)) {
+    return(NA)
+  }
+  if (any(!is.finite(sims))) {
     return(NaN)
-  else if (is_constant(sims))
-    return(1)
+  }
+  if (is_constant(sims)) {
+    return(NA)
+  }
   if (is.vector(sims)) {
     dim(sims) <- c(length(sims), 1)
   }
@@ -208,10 +216,15 @@ ess_rfun <- function(sims) {
   }
   chains <- ncol(sims)
   n_samples <- nrow(sims)
-  if (any(!is.finite(sims)) || n_samples < 3L)
+  if (n_samples < 3L || anyNA(sims)) {
+    return(NA)
+  }
+  if (any(!is.finite(sims))) {
     return(NaN)
-  else if (is_constant(sims))
-    return(chains*n_samples)
+  }
+  if (is_constant(sims)) {
+    return(NA)
+  }
   acov <- lapply(seq_len(chains), function(i) autocovariance(sims[, i]))
   acov <- do.call(cbind, acov)
   chain_mean <- apply(sims, 2, mean)
@@ -327,9 +340,9 @@ ess_bulk <- function(sims) {
 #' 
 #' @export
 ess_tail <- function(sims) {
-  I05 <- sims <= quantile(sims, 0.05)
+  I05 <- sims <= quantile(sims, 0.05, na.rm = TRUE)
   q05_ess <- ess_rfun(split_chains(I05))
-  I95 <- sims <= quantile(sims, 0.95)
+  I95 <- sims <= quantile(sims, 0.95, na.rm = TRUE)
   q95_ess <- ess_rfun(split_chains(I95))
   min(q05_ess, q95_ess)
 }
@@ -353,7 +366,7 @@ ess_tail <- function(sims) {
 #' 
 #' @export
 ess_quantile <- function(sims, prob) {
-  I <- sims <= quantile(sims, prob)
+  I <- sims <= quantile(sims, prob, na.rm = TRUE)
   ess_rfun(split_chains(I))
 }
 
@@ -576,8 +589,8 @@ monitor <- function(sims, warmup = floor(dim(sims)[1] / 2),
   for (i in seq_along(out)) {
     sims_i <- sims[, , i]
     valid <- all(is.finite(sims_i))
-    quan <- unname(quantile(sims_i, probs = probs))
-    quan2 <- quantile(sims_i, probs = c(0.05, 0.5, 0.95))
+    quan <- unname(quantile(sims_i, probs = probs, na.rm = TRUE))
+    quan2 <- quantile(sims_i, probs = c(0.05, 0.5, 0.95), na.rm = TRUE)
     mean <- mean(sims_i)
     sd <- sd(sims_i)
     mcse_quan <- sapply(probs, mcse_quantile, sims = sims_i)
@@ -594,7 +607,7 @@ monitor <- function(sims, warmup = floor(dim(sims)[1] / 2),
   }
   
   out <- as.data.frame(do.call(rbind, out))
-  probs_str <- names(quantile(sims_i, probs = probs))
+  probs_str <- names(quantile(sims_i, probs = probs, na.rm = TRUE))
   str_quan <- paste0("Q", probs * 100)
   str_quan2 <- paste0("Q", c(0.05, 0.5, 0.95) * 100)
   str_mcse_quan <- paste0("MCSE_", str_quan)
